(ns minibaldur.engine)

(defn diceroll
  [x]
  (re-find #"(\d+)d(\d+)" x))

(defn diceval
  [x]
  (if-let [[_ a b] (diceroll x)]
    (apply + (repeat (Integer/parseInt a)
                     (+ (rand-int (Integer/parseInt b)) 1)))
    (Integer/parseInt x)))

(defn roll
  "Takes a standard ttrpg roll specifier and returns a vector, starting with the sum, and 
  then each roll in the order listed in the specifier"
  [specifier]
  (let [dies-and-ops (clojure.string/split specifier #"\s+")
        first-die-val (diceval (first dies-and-ops))
        first-die-roll? (diceroll (first dies-and-ops))]
    (loop [sum first-die-val
           rolls (if first-die-roll? [first-die-val] [])
           tail (next dies-and-ops)]
      (if-let [[op die] tail]
        (let [roll? (diceroll die)
              val (diceval die)]
          (case op
            "+" (recur (+ sum val) (if roll? (conj rolls val) rolls) (nnext tail))
            "-" (recur (- sum val) (if roll? (conj rolls val) rolls) (nnext tail))))
        (into [sum] rolls)))))
